package com.link.core.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.link.base.enterprise.model.CoreEnterprise;
import com.link.base.enterprise.serivce.CoreEnterpriseService;
import com.link.base.user.model.CoreUser;
import com.link.base.user.service.CoreUserService;
import com.link.core.basic.interceptor.SecurityInterceptor;
import java.time.Duration;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class TokenUtils {
   private static Logger logger = LoggerFactory.getLogger(TokenUtils.class);
   public static final String INNER_TOKEN = "INNERTOKEN";
   public static final String USER_KEY = "user";
   public static final String ENTERPRISE = "enterprise";
   public static final String USER_JSON_KEY = "userJson";
   public static final String ENTERPRISE_JSON_KEY = "enterpriseJson";
   private static LoadingCache<String, Map<String, Object>> innerCache = CacheBuilder.newBuilder().initialCapacity(50).maximumSize(100L).expireAfterWrite(Duration.ofDays(1L)).build(new CacheLoader<String, Map<String, Object>>() {
      public Map<String, Object> load(String key) throws Exception {
         TokenUtils.logger.info("缓存登陆加载用户、企业信息，用户名：" + key);
         CoreUser user = new CoreUser();
         user.setUsername(key);
         user = ((CoreUserService)BeanTools.getApplicationContext().getBean(CoreUserService.class)).queryByUserName(user);
         if (null == user) {
            throw new Exception("无法查询到该用户");
         } else {
            CoreEnterprise enterprise = new CoreEnterprise();
            enterprise.setCode(user.getCorpid());
            enterprise = ((CoreEnterpriseService)BeanTools.getApplicationContext().getBean(CoreEnterpriseService.class)).queryByCode(enterprise);
            Map<String, Object> modelMap = new HashMap(8);
            modelMap.put("user", user);
            modelMap.put("enterprise", enterprise);
            String userEnterJson = JSON.toJSONString(modelMap);
            String innerToken = AesUtil.encrypt(userEnterJson);
            modelMap.put("INNERTOKEN", innerToken);
            String userJson = JSONObject.toJSONString(user);
            String entJson = JSONObject.toJSONString(enterprise);
            modelMap.put("userJson", userJson);
            modelMap.put("enterpriseJson", entJson);
            return modelMap;
         }
      }
   });

   public static Map<String, Object> setLogin(String userName) throws Exception {
      CoreUser user = new CoreUser();
      user.setUsername(userName);
      user = ((CoreUserService)BeanTools.getApplicationContext().getBean(CoreUserService.class)).queryByUserName(user);
      if (null == user) {
         throw new Exception("无法查询到该用户");
      } else {
         CoreEnterprise enterprise = new CoreEnterprise();
         enterprise.setCode(user.getCorpid());
         enterprise = ((CoreEnterpriseService)BeanTools.getApplicationContext().getBean(CoreEnterpriseService.class)).queryByCode(enterprise);
         Map<String, Object> modelMap = new HashMap(8);
         modelMap.put("user", user);
         modelMap.put("enterprise", enterprise);
         String userEnterJson = JSON.toJSONString(modelMap);
         String innerToken = AesUtil.encrypt(userEnterJson);
         modelMap.put("INNERTOKEN", innerToken);
         String userJson = JSONObject.toJSONString(user);
         String entJson = JSONObject.toJSONString(enterprise);
         modelMap.put("userJson", userJson);
         modelMap.put("enterpriseJson", entJson);
         SecurityInterceptor.internal_token.set(innerToken);
         SecurityInterceptor.tempUser.set(user);
         SecurityInterceptor.tempEnterpirse.set(enterprise);
         String linkRequestId = UUID.randomUUID().toString();
         ThreadContext.put("LinkRequestId", linkRequestId);
         SecurityInterceptor.linkRequestId.set(linkRequestId);
         return modelMap;
      }
   }

   public static Map<String, Object> setLogin(CoreUser user, CoreEnterprise enterprise) throws Exception {
      Map<String, Object> modelMap = new HashMap(8);
      modelMap.put("user", user);
      modelMap.put("enterprise", enterprise);
      String userEnterJson = JSON.toJSONString(modelMap);
      String innerToken = AesUtil.encrypt(userEnterJson);
      modelMap.put("INNERTOKEN", innerToken);
      String userJson = JSONObject.toJSONString(user);
      String entJson = JSONObject.toJSONString(enterprise);
      modelMap.put("userJson", userJson);
      modelMap.put("enterpriseJson", entJson);
      SecurityInterceptor.internal_token.set(innerToken);
      SecurityInterceptor.tempUser.set(user);
      SecurityInterceptor.tempEnterpirse.set(enterprise);
      String linkRequestId = UUID.randomUUID().toString();
      ThreadContext.put("LinkRequestId", linkRequestId);
      SecurityInterceptor.linkRequestId.set(linkRequestId);
      return modelMap;
   }

   public static Map<String, Object> setLoginByCache(String userName) throws Exception {
      Map<String, Object> modelMap = (Map)innerCache.get(userName);
      CoreUser user = (CoreUser)modelMap.get("user");
      CoreEnterprise enterprise = (CoreEnterprise)modelMap.get("enterprise");
      String innerToken = (String)modelMap.get("INNERTOKEN");
      SecurityInterceptor.internal_token.set(innerToken);
      SecurityInterceptor.tempUser.set(user);
      SecurityInterceptor.tempEnterpirse.set(enterprise);
      String linkRequestId = UUID.randomUUID().toString();
      ThreadContext.put("LinkRequestId", linkRequestId);
      SecurityInterceptor.linkRequestId.set(linkRequestId);
      return modelMap;
   }
}
